@@ -53,7 +53,8 @@ ngx_http_chunked_header_filter(ngx_http_request_t *r)
     if (r->headers_out.status == NGX_HTTP_NOT_MODIFIED
         || r->headers_out.status == NGX_HTTP_NO_CONTENT
         || r->headers_out.status == NGX_HTTP_CREATED
-        || r != r->main)
+        || r != r->main
+        || (r->method & NGX_HTTP_HEAD))
     {
         return ngx_http_next_header_filter(r);
     }
